package com.hand.app.zhongDengWang.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.hand.app.zhongDengWang.service.IZdwWsRequestsService;
import com.hand.app.zhongDengWang.dto.ZdwLoginTokenInterface;
import com.hand.app.zhongDengWang.dto.FndZhongDengLines;
import com.hand.app.zhongDengWang.mapper.ZdwLoginTokenInterfaceMapper;
import com.hand.app.zhongDengWang.mapper.FndZhongDengLinesMapper;
import com.hand.app.zhongDengWang.utils.ClientUtils;
import com.hand.app.zhongDengWang.ws.init.WSInitRegisterService;
import com.hand.app.zhongDengWang.ws.init.WSInitRegisterServiceService;
import com.hand.app.zhongDengWang.ws.login.WSLoginService;
import com.hand.app.zhongDengWang.ws.login.WSLoginServiceService;
import com.hand.app.zhongDengWang.dto.FndAtmAttachmentDto;
import com.hand.app.zhongDengWang.mapper.FndAtmAttachmentMapper;
import com.hand.hap.core.IRequest;
import com.rfp.sm.utils.SM2Utils;
import org.json.XML;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Service
public class ZdwWsRequestsServiceImpl implements IZdwWsRequestsService {

    @Autowired
    ZdwLoginTokenInterfaceMapper zdwLoginTokenInterfaceMapper;
    @Autowired
    FndZhongDengLinesMapper fndZhongDengLinesMapper;
    @Autowired
    FndAtmAttachmentMapper fndAtmAttachmentMapper;

    @Value("${zdw.login.clientKey}")
    private String clientKey;
    @Value("${zdw.login.userName}")
    private String userName;
    @Value("${zdw.login.password}")
    private String userPassword;
    @Value("${zdw.login.authCode}")
    private String platformAuthCode;

    private String loginToken;

    /**
     * 加密公钥 String clientKey
     * 用户名 String userName
     * 密码  String userPassword
     * 平台授权码 String platformAuthCode
     **/
    public static String clintLogin(String clientKey, String userName, String userPassword, String platformAuthCode) {
        String returnMsg = "";

        try {
            /*SM2算法加密*/
            userName = SM2Utils.encryptByKeyStr(userName, clientKey);
            userPassword = SM2Utils.encryptByKeyStr(userPassword, clientKey);
            platformAuthCode = SM2Utils.encryptByKeyStr(platformAuthCode, clientKey);
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        WSLoginServiceService service = new WSLoginServiceService();
        WSLoginService login = service.getWSLoginServicePort();
        try {
            byte[] returns = login.login(userName.getBytes(), userPassword.getBytes(), platformAuthCode.getBytes());
            returnMsg = new String(returns);
        } catch (Exception e) {
            e.printStackTrace();
            returnMsg = "LoginError";
        }
        System.out.println("token=" + returnMsg);
        return returnMsg;
    }

    private String getRegisterToken() {

        //获取token，后台设置每25分钟重置一次token
        ZdwLoginTokenInterface zdwLoginTokenInterface = new ZdwLoginTokenInterface();
        zdwLoginTokenInterface.setEnableFlag("Y");
        zdwLoginTokenInterface = zdwLoginTokenInterfaceMapper.selectOne(zdwLoginTokenInterface);

        String assesstoken = "";

        if (Objects.isNull(zdwLoginTokenInterface)) {
//            assesstoken = clintLogin(clientKey, userName, userPassword, platformAuthCode);
            assesstoken = "cs11111111";
            if (!assesstoken.contains("Error")) {
                zdwLoginTokenInterfaceMapper.updateToken(assesstoken);
            }
        } else {
            assesstoken = zdwLoginTokenInterface.getToken();
        }

        return assesstoken;
    }


    /***
     * 初始登记;
     * @param request
     * @param batchId
     ***/
    public JSONObject initRegisterService(IRequest request, Long batchId) {
        JSONObject resultJson = new JSONObject();
        List<Long> lineIds = new ArrayList<>();
        lineIds.add(batchId);
        if (lineIds.size() == 0) {
            resultJson.put("code", "000");
            resultJson.put("info", "000");
            return resultJson;
        }
        String xmlFileContent = "";
        String attachmentZipPath = "";

        /*获取中登网登录loginToken*/
        loginToken = getRegisterToken();
        /*返回结果loginToken解析，如果返回异常退出当前请求*/
        if (loginToken.contains("Error")) {
            String errorCode = "-1";
            String errorInfo = "-1";
            if (errorCode.contains(":")) {
                errorCode = loginToken.split(":")[1];
                if (errorCode.contains("_")) {
                    errorCode = loginToken.split("_")[0];
                    errorInfo = loginToken.split("_")[1];
                }
            }

            resultJson.put("code", errorCode);
            resultJson.put("info", errorInfo);
        } else {

            /*根据传入ids循环上报中登网*/
            for (Long lineId : lineIds) {

                FndZhongDengLines fndZhongDengLines = new FndZhongDengLines();
                fndZhongDengLines.setLineId(lineId);
                fndZhongDengLines = fndZhongDengLinesMapper.selectByPrimaryKey(fndZhongDengLines);

                if (Objects.nonNull(fndZhongDengLines)) {
                    try {
                        String returnMsg = "";
                        org.json.JSONObject returnJson;

                        FndAtmAttachmentDto fndAtmAttachmentDto=new FndAtmAttachmentDto();
                        fndAtmAttachmentDto.setAttachmentId(fndZhongDengLines.getAttachmentXmlId());
                        fndAtmAttachmentDto=fndAtmAttachmentMapper.selectByPrimaryKey(fndAtmAttachmentDto);

                        /*解析xml文件内容*/
                        xmlFileContent = new String(ClientUtils.getxmlFileContent(fndAtmAttachmentDto.getFilePath()));

                        /*SM2算法加密*/
                        loginToken = SM2Utils.encryptByKeyStr(loginToken, clientKey);
                        xmlFileContent = SM2Utils.encryptByKeyStr(xmlFileContent, clientKey);
                        platformAuthCode = SM2Utils.encryptByKeyStr(platformAuthCode, clientKey);

                        WSInitRegisterServiceService service = new WSInitRegisterServiceService();
                        WSInitRegisterService reg = service.getWSInitRegisterServicePort();

                        byte[] returns = reg.initRegister(fndZhongDengLines.getTypebz().getBytes(), platformAuthCode.getBytes(), loginToken.getBytes(), "lr".getBytes(), xmlFileContent.getBytes(), ClientUtils.getAttachmentZip(attachmentZipPath));
                        returnMsg = new String(returns);
                        returnJson = XML.toJSONObject(returnMsg);

                        System.out.println("dddd" + new String(returns));
                    } catch (Exception e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }


            }

        }
        return resultJson;
    }
}